import basicOperations 
import globalFunctions as gf
import matrixOperations as mo

def mult(a, b):
    if all(isinstance(x, list) for x in a) and all(isinstance(x, list) for x in b):
        return basicOperations.matrixMultiply(a,b)
    else:
        gf.errorTraceback()
        gf.printError('List of lists not found for matrices A and B')

# Working multiplication
#print(mult([[1,2]], [[3,4], [5,6]]))
# Intential Error:
#print(mult([1,2], [[3,4]]))

def add(a, b):
    if all(isinstance(x, list) for x in a) and all(isinstance(x, list) for x in b):
        return basicOperations.matrixAdd(a,b)
    else:
        gf.errorTraceback()
        gf.printError('List of lists not found for matrices A and B')

def sub(a, b):
    if all(isinstance(x, list) for x in a) and all(isinstance(x, list) for x in b):
        return basicOperations.matrixSub(a,b)
    else:
        gf.errorTraceback()
        gf.printError('List of lists not found for matrices A and B')

# Working addition
#print(add([[1,2], [7,8]], [[3,4], [5,6]]))
# Intential Error:
#print(sub([[1,2] , [1,1]], [[3,4] , [5,6]]))

def determinent(a):
    if all(isinstance(x, list) for x in a):
        return mo.determinent(a)
    else:
        gf.errorTraceback()
        gf.printError('List of lists not found for the matrix')

# Working 
#print(determinent([[1,2] , [3,4]]))
# Failure
#print(determinent([[1,2]]))

def transpose(a):
    if all(isinstance(x, list) for x in a):
        return mo.transposeMatrix(a)
    else:
        gf.errorTraceback()
        gf.printError('List of lists not found for the matrix')

# Working 
# print(transpose([[1,2,6,7] , [3,4,5,5], [5,6,3,2]]))

def inverse(a):
    if all(isinstance(x, list) for x in a):
        return mo.inverseMatrix(a)
    else:
        gf.errorTraceback()
        gf.printError('List of lists not found for the matrix') 

# Working 
#print(inverse([[1,2,3] , [3,4,6], [3,4,5]]))




import globalFunctions as gf

def matrixMultiply(a, b):

    if len(b) == len(a[0]):

        result = [[0 for x in range(len(b[0]))] for y in range(len(a))]
        
        for i in range(len(a)):
            for j in range(len(b[0])):
                result[i][j] = 0
                for x in range(len(b[0])):           
                    result[i][j] += (a[i][x] * b[x][j])

        return result
    
    else:
        gf.errorTraceback()
        gf.printError('Columns of first matrix must match rows of second matrix!')

def matrixAdd(a, b):

    if gf.arrDim(a) == gf.arrDim(b):

        result = []
        for i in range(len(a)):
            appendList = []
            for k in range(len(a[0])):
                appendList.append(a[i][k] + b[i][k])
            result.append(appendList)

        return result
    else:
            gf.errorTraceback()
            gf.printError('Both matrices must have the same dimensions!')

def matrixSub(a, b):

    if gf.arrDim(a) == gf.arrDim(b):

        result = []
        for i in range(len(a)):
            appendList = []
            for k in range(len(a[0])):
                appendList.append(a[i][k] - b[i][k])
            result.append(appendList)

        return result
    else:
            gf.errorTraceback()
            gf.printError('Both matrices must have the same dimensions!')


import traceback
import sys

def errorTraceback():
    for line in traceback.format_stack():
        print(line.strip())

def printError(str):
    print('\n' + str + '\n', file=sys.stderr)
    sys.exit(1)

def arrDim(a):
  return [len(a)] + arrDim(a[0]) if(type(a) == list) else []


from sympy import det
import globalFunctions as gf

def __cofactor(a, i , j):
    return [row[: j] + row[j+1:] for row in (a[: i] + a[i+1:])]

def determinent(a):

    dim = gf.arrDim(a)

    if (dim[0] == dim[1]):
        if dim[0] == 2:
            deter = a[0][0] * a[1][1] - a[1][0] * a[0][1]
            return deter
        else: 
            deter = 0 

            for i in range(dim[0]):
                sign = (-1) ** i

                deterInner = determinent(__cofactor(a, 0, i))
                deter += (sign * a[0][i] * deterInner)
            
            return deter
    else:
        gf.errorTraceback()
        gf.printError('Matrix is not a square matrix!')

def transposeMatrix(a):

    transposed = [[0 for x in range(len(a))] for y in range(len(a[0]))]

    for i in range(len(a[0])):
        for j in range(len(a)):
            transposed[i][j] = a[j][i]

    return transposed

def inverseMatrix(a):
    dim = gf.arrDim(a)

    if (dim[0] == dim[1]):

        deter = determinent(a)
        if (deter != 0):
            inverse = [[0 for x in range(len(a[0]))] for y in range(len(a))]

            if (len(inverse) == 1):
                inverse[0][0] = 1
            else:
                sign = 1

                for i in range(len(a)):
                    for j in range(len(a)):
                        cofacter = [[0 for x in range(len(a[0]))] for y in range(len(a))]
                        cofacter = __cofactor(a, i, j)
                        
                        if (i + j) % 2 == 0:
                            sign = 1
                        else:
                            sign = -1
                        
                        inverse[j][i] = (sign) * (determinent(cofacter))
            
            for i in range(len(a)):
                for j in range(len(a)):
                    inverse[i][j] = inverse[i][j] / deter

            return inverse
        else:
            gf.errorTraceback()
            gf.printError('Singular matrix, no inverse!')
    else:
        gf.errorTraceback()
        gf.printError('Matrix is not a square matrix!')
    
def systemOfEquations(a):

    #sol = [0 for x in range(len(a) - 1)]
    



from setuptools import setup, find_packages
import codecs
import os

VERSION = '0.0.1'
DESCRIPTION = 'Linear Algebra Calculator with simple functions'
#LONG_DESCRIPTION = 'A linear algebra function that uses python to calculate values such as the inverse or transpose of a matrix and much more'

# Setting up
setup(
    name="linearAlgebraVectorMath",
    version=VERSION,
    author="Aditya Srikanth",
    author_email="aditya.srikanth11@gmail.com" ,
    description=DESCRIPTION,
    #long_description_content_type="text/markdown",
    #long_description=long_description,
    packages=find_packages(),
    install_requires=['traceback'],
    keywords=['python', 'mathamatics', 'Linear Algebra', 'Vectors', 'Eigen Values'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)